#include "lua.hpp"
#include <string.h>

static int fun0(lua_State* L)
{
  // stack:
  printf("fun0\n");

  return 0;
}

static int fun1(lua_State* L)
{
  // stack: string
  const char* text = lua_tostring(L, 1);
  printf("fun1 argument[%s]\n", text);
  lua_pushnumber(L, strlen(text)); // stack: string number

  return 1;
}

static int fun2(lua_State* L)
{
  // stack: number string
  double number = lua_tonumber(L, 1);
  const char* text = lua_tostring(L, 2);
  printf("fun2 arg1[%lf] arg2[%s]\n", number, text);

  lua_pushstring(L, text); // stack: number string text
  lua_pushnumber(L, number); // stack: number string text number

  return 2;
}

static int funVar(lua_State* L)
{
  // stack: ...
  const int count = lua_gettop(L);
  printf("funVar arguments count %d\n", count);
  for(int i = 1; count >= i; ++i)
  {
    printf("funVar argument %d is ", i);
    if(lua_isnumber(L, i))
    {
      printf("%lf\n", lua_tonumber(L, i));
    }
    else if(lua_isstring(L, i))
    {
      printf("%s\n", lua_tostring(L, i));
    }
    else
    {
      printf("invalid argument type\n");
    }
  }

  lua_pushnumber(L, count); // stack: ... count

  return 1;
}

static const luaL_Reg test[] = {
  {"Fun0", fun0},
  {"Fun1", fun1},
  {"Fun2", fun2},
  {"FunVar", funVar},
  {0, 0}
};

extern "C" __declspec(dllexport) int luaopen_test(lua_State* L)
{
  luaL_register(L, "test", test);
  return 1;
}
